package org.jeecg.modules.app.controller;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.modules.app.dto.UserWechatRegisterDTO;
import org.jeecg.modules.app.service.IUserService;
import org.jeecg.modules.app.vo.LoginUserVO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@Slf4j
@Tag(name = "App用户相关接口")
@RestController
@RequestMapping("/app/user")
public class AppUserController {
    @Resource
    private IUserService userService;


    /**
     * 登录
     *
     * @param code
     * @return {@link Result}<{@link ?}>
     */
    @GetMapping("/loginByCode")
    @Operation(summary = "微信小程序登录接口")
    public Result<LoginUserVO> wechatLogin(@RequestParam @Schema(description = "微信登录返回的code") String code){
        return Result.ok(userService.wechatLogin(code));
    }

    /**
     * 微信小程序获取用户信息接口
     *
     * @param request               请求
     * @param userWechatRegisterDTO {@link UserWechatRegisterDTO}
     * @return {@link Result}<{@link ?}>
     */
    @PostMapping("/wechatRegister")
    @Operation(summary = "微信小程序获取用户信息接口")
    public Result<LoginUserVO> wechatInfo(@RequestBody UserWechatRegisterDTO userWechatRegisterDTO, HttpServletRequest request){
        return Result.ok(userService.wechatInfo(userWechatRegisterDTO, request));
    }
}
